#!/bin/sh

vchans=`sysctl -n hw.snd.pcm0.vchans`
if [ ${vchans} -ne 0 ] ; then
    echo "This test needs to be run without vchans enabled and preferably on"
    echo "hardware that supports 8, 44.1, and 48kHz sampling."
    exit 1
fi

# ignore dd output
exec 2>/dev/null

BLKSZ=512
TONESECS=4

get_channels()
{
    echo $1
}

get_src_rate()
{
    echo $2
}

get_dst_rate()
{
    echo $3
}

get_count()
{
    local channels rate cnt
    channels=$1
    rate=$2
    tsecs=$3
    cnt=`echo "${rate} ${channels} * 2 * ${tsecs} * ${BLKSZ} / p" | dc`
    echo $cnt
}

for conversion in 							      \
	"2 8000 44100" "2 8000 44095" "2 16000 8000" "2 44100 8000"	      \
; do
    c=`get_channels $conversion`
    s=`get_src_rate $conversion`
    d=`get_dst_rate $conversion`

    echo "------------------------------------------------------------"
    echo "Tone at destination rate $d"
    blks=`get_count $c $d ${TONESECS}`
    ./tonegen -s conv -c $c 414 $d | dd count=${blks} | ./rawplay -c $c $d

    sleep 1

    echo "Tone at source rate $s converted to $d with old converter"
    blks=`get_count $c $s ${TONESECS}`
    ./tonegen -s conv -c $c 414 $s | dd count=${blks} | \
	./feeder_rate_fbsd -c $c $s $d | \
	./rawplay -c $c $d

    sleep 1

    echo "Tone at source rate $s converted to $d with new converter"
    ./tonegen -s conv -c $c 414 $s | dd count=${blks} | \
	./feeder_rate_new -c $c $s $d | \
	./rawplay -c $c $d

    sleep 1

    sleep 3
done
