#!/bin/sh

convert_for_time()
{
    local feeder pause pid

    feeder=$1
    pause=$2
    src=$3
    dst=$4
    ofile=$5

    $feeder -n -c 2 $src $dst 1>$ofile 2>&1 &

    pid=$!
    sleep $pause

    kill -SIGINT $pid
    wait $pid

    # wait for file to be written to - p is for plop
    tries=0
    while [ ! -s $ofile ] ; do
	tries=`expr $tries + 1`
	if [ $tries -gt 3 ] ; then
	    echo "no file or empty file"
	    exit
	fi
	sleep 1
    done
}

for c in "48000 44100" "44100 8000" "8000 44100" "44100 48000" ; do
    exp_ratio=`echo $c | awk '{ printf "%.4f", $1 / $2; }'`
    echo "###############################################################################"
    echo "#"
    echo "# Converting $c expecting <bytes in> / <bytes out> $exp_ratio"
    echo "# Column 1 - program"
    echo "# Column 2 - execution time"
    echo "# Column 3 - bytes accepted as input"
    echo "# Column 4 - bytes generated as output"
    echo "# Column 5 - bytes_input / bytes_output"
    echo "# Column 6 - %realtime"
    echo "#"
    inrate=`echo $c | awk '// { print $1}'`
    outrate=`echo $c | awk '// { print $2}'`
    for feeder_rate in ./feeder_rate_fbsd ./feeder_rate_new ; do
	for s in 1 2 3 ; do
	    ofile=/tmp/draft_watch-$$.dat
	    convert_for_time $feeder_rate $s $inrate $outrate $ofile
	    inbytes=`cat $ofile | awk '/read/ { print $3 }'`
	    outbytes=`cat $ofile | awk '/wrote/ { print $3 }'`
	    ratio=`dc -e "4k $inbytes $outbytes / p"`
	    realtime=`dc -e "2k 100 $inbytes * $inrate 4 $s * * / p"`
	    echo $feeder_rate $s $inbytes $outbytes $ratio $realtime
	    rm -f $ofile
	done
	echo
    done
done