#!/bin/sh

sysctl hw.snd.pcm0.vchans=4
if [ $? -ne 0 ] ; then
   echo "Got root ? Failed to enable vchans."
   exit 1
fi

trap 'kill 0; exit 1' exit SIGINT

# Start playing a tone with an 8kHz sample rate
./tonegen -a 16383 -c 2 262 8000 | ./rawplay -f /dev/audio0.1 -c 2 8000 &

# Start recording signal with a 17kHz sample rate
./rawrec -f /dev/audio0.2 -c 1 17000 > /dev/null &

# Cycle through different combinations of block sizes, channels and sample
# rates playing a tone.
#
# The tone duration is set with:
TSECS=10

for BLKSZ in 128 256 512 1024 2048; do
    for channels in 1 2 ; do
	for rate in 8000 8888 11025 16000 22050 23000 32000 44100 48000 ; do
	    printf "Playing channels %d rate %d..." $channels $rate
	    cnt=`echo "${rate} ${channels} * 2 * ${TSECS} * ${BLKSZ} / p" | dc`
	    ./tonegen -s conv -a 16383 -c ${channels} 414 ${rate} | \
		    dd bs=${BLKSZ} count=${cnt} | \
		    ./rawplay -f /dev/audio0.0 -B ${BLKSZ} -c ${channels} ${rate} 2>/dev/null
	    echo "done"
	    sleep 1
	done
    done
done

kill ${BPID}

