/*
 * There is a lot of scrap here.  It's not very well organized junk to
 * provide minimal "emulation" of the kobj code for feeder objects.
 */

struct pcm_channel;
struct pcm_feeder;

#ifndef UNUSED
#define UNUSED(x) (x) = (x)
#endif

/* Types for function pointers */

typedef int (*feed_init_fn)(struct pcm_feeder *f);
typedef int (*feed_free_fn)(struct pcm_feeder *f);
typedef int (*feed_set_fn)(struct pcm_feeder *f, int what, int value);
typedef int (*feed_get_fn)(struct pcm_feeder *f, int what);
typedef int (*feed_feed_fn)(struct pcm_feeder *f, struct pcm_channel* c,
			    uint8_t *feed_buffer, uint32_t feed_bytes,
			    void *source);

typedef int (*feed_eof_fn)(struct pcm_feeder* f);

typedef struct {
    const char* name;
    void *method;
} kobj_method_t;

#define KOBJMETHOD(x, y) { #x, y }

struct pcm_feederdesc {
    uint32_t type;
    uint32_t from;
    uint32_t to;
    uint32_t align;
};

#define FEEDER_RATE 98
#define FEEDER_SINC 99
#include <machine/soundcard.h>

struct pcm_feeder {
	void *data;
	struct pcm_feeder *source;
	feed_init_fn init;
	feed_free_fn free;
	feed_set_fn  set;
	feed_get_fn  get;
	feed_feed_fn feed;
	feed_eof_fn  eof;
};

struct feeder_class {
	const char*		name;
	kobj_method_t*		methods;
	size_t			size;
	int			align;
	struct pcm_feederdesc 	*desc;
	void 			*data;
};

#define FEEDER_DECLARE(feeder, palign, pdata) \
\
static struct feeder_class feeder ## _class = { \
	name:		#feeder, \
	methods:	feeder ## _methods, \
	size:		sizeof(struct pcm_feeder), \
	align:		palign, \
	desc:		feeder ## _desc, \
	data:		pdata, \
};

static void*
class_get_method(struct feeder_class *cls, const char* name)
{
    kobj_method_t * m = cls->methods;
    while (m->name) {
	if (strcmp(m->name, name) == 0)
	    return m->method;
	m++;
    }
    return NULL;
}

static struct pcm_feeder* feeder_create(struct feeder_class *cls)
{
    struct pcm_feeder *f = (struct pcm_feeder *)malloc(sizeof(*f));

    f->source = NULL;
    f->data = NULL;
    
    f->init = (feed_init_fn)class_get_method(cls, "feeder_init");
    f->free = (feed_free_fn)class_get_method(cls, "feeder_free");
    f->set = (feed_set_fn)class_get_method(cls, "feeder_set");
    f->get = (feed_get_fn)class_get_method(cls, "feeder_get");
    f->feed = (feed_feed_fn)class_get_method(cls, "feeder_feed");
    f->eof = (feed_eof_fn)class_get_method(cls, "feeder_eof");        

    if (f->init && f->init(f)) {
	free(f);
	return NULL;
    }	
    return f;
}

static void
feeder_destroy(struct pcm_feeder *f)
{
    if (f->free)
	f->free(f);
}

#define FEEDER_CREATE(feeder) feeder_create(& feeder ## _class)
#define FEEDER_DESTROY(feeder) feeder_destroy(feeder)

static int
FEEDER_FEED(struct pcm_feeder	*f, 
	    struct pcm_channel	*c, 
	    uint8_t		*b,
	    uint32_t		 count, 
	    void		*source)
{
    return f->feed(f, c, b, count, source);
}

int FEEDER_SET(struct pcm_feeder *, int, int);
int FEEDER_GET(struct pcm_feeder *, int);
int FEEDER_EOF(struct pcm_feeder *);

int
FEEDER_SET(struct pcm_feeder	*f,
	   int			 what,
	   int			 value)
{
    if (f->set)
	return f->set(f, what, value);
    return -1;
}

int
FEEDER_GET(struct pcm_feeder	*f,
	   int			 what)
{
    if (f->get)
	return f->get(f, what);
    return -1;
}

int
FEEDER_EOF(struct pcm_feeder *f)
{
    while (f != NULL) {
	if (f->eof)
	    return f->eof(f);
	f = f->source;
    }
    return 0;
}

static struct pcm_feeder*
FEEDER_JOIN(struct pcm_feeder *src, struct pcm_feeder *dst)
{
    dst->source = src;
    return dst;
}

#ifndef M_NOWAIT
#define M_NOWAIT 0
#endif

#ifndef M_ZERO
#define M_ZERO 1
#endif

#ifndef DSP_DEFAULT_SPEED
#define DSP_DEFAULT_SPEED 8000
#endif

#define MALLOC_DEFINE(x, y, z) static const int x = 0;
#define malloc(x, y, z) malloc(x)
#define free(x, y) free(x)

#ifndef min
#define min(a,b) (((a) < (b)) ? (a) : (b))
#endif /* min */

#include <assert.h>
#define RATE_ASSERT(x, y) assert(x)
