#ifndef __FEEDER_FINITE_C__

/*****************************************************************************/
/* Finite null input feeder */

static uint64_t feed_finite_bytes = 0;
#define BYTE_COUNT	0
#define BYTE_MAX	1

static void
feed_finite_atexit(void)
{
    fprintf(stderr, "feed_finite read %llu bytes\n", feed_finite_bytes);
}

static int
feed_finite_init(struct pcm_feeder *f)
{
    int32_t *bytes;
    atexit(feed_finite_atexit);

    bytes = (int32_t*)malloc(2 * sizeof(int32_t), dummy, dummy);
    if (bytes == NULL)
	return -1;
    f->data = bytes;

    bytes[BYTE_COUNT] = 0;
    bytes[BYTE_MAX] = 0x7fffffff;
    
    return 0;
}

static int
feed_finite_free(struct pcm_feeder *f)
{
    if (f->data)
	free(f->data, dummy);
    f->data = NULL;
    return 0;
}

static int
feed_finite(struct pcm_feeder	*f, 
	    struct pcm_channel	*c, 
	    uint8_t		*b,
	    uint32_t		 count, 
	    void		*source)
{
    uint32_t *bytes = f->data;
    uint32_t avail;
    avail = bytes[BYTE_MAX] - bytes[BYTE_COUNT];

    if (avail < count) 
	count = avail;

    count = FEEDER_FEED(f->source, c, b, count, source);
    feed_finite_bytes += count;
    bytes[BYTE_COUNT] += count;

    return count;
}

static int
feed_finite_eof(struct pcm_feeder *f)
{
    uint32_t *bytes = f->data;
    return bytes[BYTE_COUNT] >= bytes[BYTE_MAX];
}

#define FEEDER_FINITE_MAXBYTES 1
static int
feed_finite_set(struct pcm_feeder *f, int what, int value)
{
    int32_t* bytes = f->data;

    switch(what) {
    case FEEDER_FINITE_MAXBYTES:
	bytes[BYTE_MAX] = value;
	break;
    default:
	return -1;
    }
    return 0;
}

static int
feed_finite_get(struct pcm_feeder *f, int what)
{
    int32_t* bytes = f->data;

    switch(what) {
    case FEEDER_FINITE_MAXBYTES:
	return bytes[BYTE_MAX];
    default:
	return -1;
    }
    return -1;
}

static struct pcm_feederdesc feeder_finite_desc[] = {
    {0, 0, 0, 0}
};

static kobj_method_t feeder_finite_methods[] = {
    KOBJMETHOD(feeder_init, feed_finite_init),
    KOBJMETHOD(feeder_free, feed_finite_free),
    KOBJMETHOD(feeder_feed, feed_finite),
    KOBJMETHOD(feeder_set,  feed_finite_set),
    KOBJMETHOD(feeder_get,  feed_finite_get),
    KOBJMETHOD(feeder_eof,  feed_finite_eof),
    { 0, 0 }
};

FEEDER_DECLARE(feeder_finite, 1, 0);

#endif /* __FEEDER_FINITE_C__ */
