#ifndef __FEEDER_INFINITE_C__

/*****************************************************************************/
/* Infinite null input feeder */

static uint64_t feed_nullin_bytes = 0;

static void
feed_nullin_atexit(void)
{
    fprintf(stderr, "feed_nullin read %llu bytes\n", feed_nullin_bytes);
}

static int
feed_nullin_init(struct pcm_feeder *f)
{
    atexit(feed_nullin_atexit);
    UNUSED(f);
    return 0;
}

static int
feed_nullin(struct pcm_feeder	*f, 
	  struct pcm_channel	*c, 
	  uint8_t		*b,
	  uint32_t		 count, 
	  void			*source)
{
    feed_nullin_bytes += count;

    UNUSED(f); UNUSED(c); UNUSED(source); UNUSED(b);

    return count;
}

static struct pcm_feederdesc feeder_nullin_desc[] = {
    {0, 0, 0, 0}
};

static kobj_method_t feeder_nullin_methods[] = {
    KOBJMETHOD(feeder_init, feed_nullin_init),
    KOBJMETHOD(feeder_feed, feed_nullin),
    { 0, 0 }
};

FEEDER_DECLARE(feeder_nullin, 1, 0);

/*****************************************************************************/
/* Infinite null output feeder */

static uint64_t feed_nullout_bytes = 0;

static void
feed_nullout_atexit(void)
{
    fprintf(stderr, "feed_nullout wrote %llu bytes\n", feed_nullout_bytes);
}

static int
feed_nullout_init(struct pcm_feeder *f)
{
    atexit(feed_nullout_atexit);
    UNUSED(f);
    return 0;
}

static int
feed_nullout(struct pcm_feeder	*f, 
	     struct pcm_channel	*c, 
	     uint8_t		*b,
	     uint32_t		 count, 
	     void		*source)
{
    uint8_t buf[512];
    size_t  done;
    done = FEEDER_FEED(f->source, c, buf, 512, source);
    UNUSED(b);
    UNUSED(count);
    feed_nullout_bytes += done;
    return done;
}

static struct pcm_feederdesc feeder_nullout_desc[] = {
    {0, 0, 0, 0}
};

static kobj_method_t feeder_nullout_methods[] = {
    KOBJMETHOD(feeder_init, feed_nullout_init),
    KOBJMETHOD(feeder_feed, feed_nullout),
    { 0, 0 }
};

FEEDER_DECLARE(feeder_nullout, 1, 0);

#endif /* __FEEDER_INFINITE_C__ */
