#ifndef __FEEDER_STDIO_C__

/*****************************************************************************/
/* stdin feeder */

static uint64_t feed_stdin_bytes = 0;

static void
feed_stdin_atexit(void)
{
    fprintf(stderr, "feed_stdin read %llu bytes\n", feed_stdin_bytes);
}

static int
feed_stdin_init(struct pcm_feeder *f)
{
    atexit(feed_stdin_atexit);
    UNUSED(f);
    return 0;
}

static int
feed_stdin(struct pcm_feeder	*f, 
	   struct pcm_channel	*c, 
	   uint8_t		*b,
	   uint32_t		 count, 
	   void			*source)
{
    size_t got = fread(b, 4, count / 4, stdin);
    feed_stdin_bytes += got;

    UNUSED(f); UNUSED(c); UNUSED(source);

    return (int)got * 4;
}

static int
feed_stdin_eof(struct pcm_feeder *f)
{
	UNUSED(f);
	return feof(stdin);
}

static struct pcm_feederdesc feeder_stdin_desc[] = {
    {0, 0, 0, 0}
};

static kobj_method_t feeder_stdin_methods[] = {
    KOBJMETHOD(feeder_init, feed_stdin_init),
    KOBJMETHOD(feeder_feed, feed_stdin),
    KOBJMETHOD(feeder_eof,  feed_stdin_eof),
    { 0, 0 }
};

FEEDER_DECLARE(feeder_stdin, 1, 0);

/*****************************************************************************/
/* stdout feeder */

static uint64_t feed_stdout_bytes = 0;

static void
feed_stdout_atexit(void)
{
    fprintf(stderr, "feed_stdout wrote %llu bytes\n", feed_stdout_bytes);
}

static int
feed_stdout_init(struct pcm_feeder *f)
{
    atexit(feed_stdout_atexit);
    UNUSED(f);
    return 0;
}

static int
feed_stdout(struct pcm_feeder	*f, 
	    struct pcm_channel	*c, 
	    uint8_t		*b,
	    uint32_t		 count, 
	    void		*source)
{
    uint8_t buf[512];
    size_t  done;
    done = FEEDER_FEED(f->source, c, buf, 512, source);
    if (done > 0) {
	done = fwrite(buf, 1, done, stdout);
    }
    UNUSED(b);
    UNUSED(count);
    feed_stdout_bytes += done;
    return done;
}

static struct pcm_feederdesc feeder_stdout_desc[] = {
    {0, 0, 0, 0}
};

static kobj_method_t feeder_stdout_methods[] = {
    KOBJMETHOD(feeder_init, feed_stdout_init),
    KOBJMETHOD(feeder_feed, feed_stdout),
    { 0, 0 }
};

FEEDER_DECLARE(feeder_stdout, 1, 0);

#endif  /* __FEEDER_STDIO_C__ */
