#include <sys/soundcard.h>

#include <fcntl.h>
#include <inttypes.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <sysexits.h>
#include <unistd.h>

static void
usage(void)
{
        fprintf(stderr, 
		"rawplay [-b bits per sample] [ -c <channels> ] [ -B blocksize ]"
		"<smpl_rate>");
        exit(EX_USAGE);
}

static void
play_audio(int32_t fd, int32_t blksz)
{
	int16_t* sbuf;
	size_t got;

	sbuf = (int16_t*)malloc(blksz);

	while ((got = fread(sbuf, 1, blksz, stdin)) > 0) {
	    if (write(fd, sbuf, got) < 0) {
		perror("write");
		break;
	    }
	}
	free(sbuf);
}

static int
log_base2(int n)
{
    int u = 0;
    while (n >> u) {
	u++;
    }
    return u;
}

int main(int argc, char * const * argv)
{
	const char* device;
	int channels, srate, blksz, ch, fd, frag, bits;

	snd_chan_param scp;

	device = "/dev/dsp";
	channels = 1;
	blksz = 512;
	bits = 16;

	while ((ch = getopt(argc, argv, "b:B:c:f:n:")) != -1) {
                switch (ch) {
		case 'b':
			bits = atoi(optarg);
			break;
		case 'B':
			blksz = atoi(optarg);
			break;
                case 'c':
                        channels = atoi(optarg);
                        break;
		case 'f':
			device = optarg;
			break;
                case '?':
                default:
                        usage();
                }
	}

	argc -= optind;
	argv += optind;

	if (argc != 1)
                usage();

	srate = atoi(argv[0]);
	if (srate < 8000 || srate > 48000) {
		fprintf(stderr, 
			"rate must lie in range 8000-48000\n");
		usage();
	}


	fd = open(device, O_WRONLY);
	if (fd < 0) {
		perror(device);
		exit(EX_OSFILE);
	}

	memset(&scp, 0, sizeof(scp));
	scp.play_rate = srate;

	switch (bits) {
	case 8:
		scp.play_format = AFMT_S8;
		break;
	case 16:
		scp.play_format = AFMT_S16_LE;
		break;
	default:
		fprintf(stderr, "bits per sample must be 8 or 16\n");
		usage();
	}

	if (channels == 2) 
		scp.play_format |= AFMT_STEREO;

	if (ioctl(fd, AIOSFMT, &scp) < 0) {
		perror("AIOSFMT");
		exit(EX_DATAERR);
	}

	frag = log_base2(blksz) | 0xffff0000;
	
	if (ioctl(fd, SNDCTL_DSP_SETFRAGMENT, &frag) < 0) {
		perror("SNDCTL_DSP_SETFRAGMENT");
		exit(EX_DATAERR);
	}

	play_audio(fd, blksz);

	return 0;
}
