#include <assert.h>
#include <inttypes.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <sysexits.h>
#include <unistd.h>

static void
usage(void)
{
        fprintf(stderr, "rms [-b <bits_per_sample> ] [ -c <channels> ] [ -t ms_audio ] <smpl_rate>");
        exit(EX_USAGE);
}

static void
rms(int w_samples, int channels, int bits, int srate)
{
	double r[2];
	int16_t* sb;
	int8_t* sb8;
	size_t cur;
	int i, j;
	uint32_t elapsed = 0;
	size_t got, fill;

	sb = calloc(2 * w_samples * channels, bits / 8);
	sb8 = (int8_t*)sb;

	fill = channels * w_samples * bits / 8;
	do {
		cur = 0; got = 0;
		while (got != fill &&
		       (cur = fread(sb8 + got, 1, fill - got, stdin)) > 0) {
			got += cur;
		}

		if (bits == 8) {
				/* convert from 8 bit to 16 bit */
			int8_t* sb8 = (int8_t*)sb;
			for (i = got - 1; i >= 0; i--) {
				sb[i] = sb8[i] << 8;
			}
		} else {
			got /= 2;
		}
		r[0] = r[1] = 0.0;

		for (i = 0; i < (int)got; i += channels) {
			for (j = 0; j < channels; j++) {
				double t = (sb[i + j]) / 32768.0;
				r[j] += t * t;
			}
		}
		r[0] = sqrt(channels * r[0] / (double)got);
		r[1] = sqrt(channels * r[1] / (double)got);
		printf("%8.4f %.3f %.3f\n",
		       (double)elapsed / (double)srate, r[0], r[1]);
		elapsed += got / channels;
	} while (cur != 0);

	free(sb);
}

int main(int argc, char * const * argv)
{
	int channels, srate, ch, bits, t, w_samples;

	channels = 1;
	bits	 = 16;
	t        = 100;
	while ((ch = getopt(argc, argv, "b:c:t:")) != -1) {
                switch (ch) {
		case 'b': 
			bits = atoi(optarg);
			break;
                case 'c':
                        channels = atoi(optarg);
                        break;
		case 't':
			t = atoi(optarg);
			break;
                case '?':
                default:
                        usage();
                }
	}

	argc -= optind;
	argv += optind;

	if (argc != 1)
                usage();

	if (bits != 8 && bits != 16) {
		fprintf(stderr, "bits must be 8 or 16.");
		usage();
	}

        srate = atoi(argv[0]);
	if (srate == 0 || srate > 48000) {
                fprintf(stderr, "smpl_rate must be in range 1--48000");
                usage();
        }

	w_samples = t * srate / 1000;
	if (w_samples == 0) {
		fprintf(stderr, "window size must correspond to >1ms");
	}

	rms(w_samples, channels, bits, srate);

	return 0;
}



