function sigma(N,k)
{
	v = 0.0;
	if (k == 0) {
		v = 1;
	} else if (k <= N) {
		x = pi * (1.0 * k)/(1.0 * N);
		v = sin (x) / x;
	}
	return v;
}

function generate_sigma_tables()
{
	# Do not go above 15 without changing type info
	sigma_precision = 16; 

	# Determine gain from precision, E_NO_ROLL_OPERATOR :-)
	gain = 1;
	for (i = 1; i < sigma_precision; i++) {
		gain *= 2;
	}
	gain -= 1;

	printf("static const uint8_t sigma_precision = %d;\n\n",
	       sigma_precision);
	printf("#define SIGMA_MAX_WIDTH %d\n\n", sigma_max);

	# Calculate and pretty print sigma coefficients
	for (i = sigma_min; i <= sigma_max; i += sigma_step) {
		printf ("static const int16_t sigma_%d[] = {\n\t", i);
		for (j = 0; j < i; j++) {
			printf ("%5d", sigma(i, j) * gain);
			if (j != i - 1)
				printf (",");
			if ((j + 1) % 8 == 0 && (j + 1) != i)
				printf ("\n\t");
			else printf ("\t");
		}
		printf("\n};\n\n");
	}

	# Dump sigma coefficients into structured table
	printf("static struct sigma_entry {\n");
	printf("\tuint16_t ncoeff;\n");
	printf("\tconst int16_t* coeff;\n");
	printf("} const sigma_tab[] = {\n");
	for (i = sigma_min; i <= sigma_max; i += sigma_step) {
		printf("\t {% 2d, sigma_%d },\n", i, i);
	}
	printf("};\n\n");
	print ("#define SIGMA_TABLE_SIZE (sizeof(sigma_tab)/sizeof(sigma_tab[0]))\n");
}

function sinc(x)
{
	if (x != 0) {
		return sin(x * pi) / (x * pi)
	}
	return 1;
}

function generate_sinc_tables(x_max)
{
	f_scale = 100;
	f_step = 3;
	f_max  = 0.5 * f_scale;

	gain = 32767;
	printf ("static const int32_t sinc_f_scale = %d; \n", f_scale);
	printf ("static const int32_t sinc_f_step = %d; \n\n", f_step);

	print  ("static size_t sinc_cut_off(int32_t f_lo, int32_t f_hi)");
	print  ("{");
	print  ("\treturn (size_t)(sinc_f_scale * f_lo / (2 * f_hi));");
	print  ("}\n");

	print  ("static size_t sinc_table_index(int32_t f_lo, int32_t f_hi)");
	print  ("{");
	print  ("\tsize_t index = sinc_cut_off(f_lo, f_hi) / sinc_f_step;")
	print  ("\treturn (index > 1) ? index - 1 : 0;");
	print  ("}\n");

	printf ("static struct sinc_entry {\n");
	printf ("\tint16_t f;\n");
	printf ("\tint16_t coeff[%d];\n", x_max);
	printf ("} sinc_tab[] = {\n");

	for (f = f_step; f < f_max; f += f_step) {
		printf("\t{\n");
		printf("\t\t%d,\n\t\t{\n\t\t", f);
		for (x = 0; x < x_max; x++) {
			printf("\t%6d", sinc(x * f / (1.0 * f_max)) * gain);
			if (x != x_max - 1)
				printf (",");
			if ((x + 1) % 4 == 0 && (x + 1) != x_max)
				printf ("\n\t\t");
		}
		printf("\n\t\t}");
		printf("\n\t},\n");
	}
	printf("\n};\n");
	printf("#define FEEDSINC_SRC 1\n");
	printf("#define FEEDSINC_DST 2\n\n");
	return;
}

BEGIN {
	filter_input = 0;
	sigma_min = 2;
	sigma_max = 36;
	sigma_step = 4;
	pi = 3.141592653;
}

/GENERATED \*\/$/ { 
	print; 
	filter_input = !filter_input; 
	if (filter_input) {
		generate_sigma_tables();
		generate_sinc_tables(sigma_max)
	}
	next 
}

// {
	if (!filter_input) 
		print;
}
