#include <assert.h>
#include <inttypes.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <sysexits.h>
#include <unistd.h>

#define IN_SAMPLES 64

static void
usage(void)
{
        fprintf(stderr, "timeline [-b <bits_per_sample> ] [ -c <channels> ] <smpl_rate>");
        exit(EX_USAGE);
}

static void
timeline(uint16_t srate, uint16_t channels, uint16_t bps)
{
	int16_t sbuf[IN_SAMPLES];
	
	uint32_t t;
	int16_t i;
	uint16_t c;
	size_t got;

	t = 0;
	while ((got = fread(sbuf, bps / 8, IN_SAMPLES, stdin)) > 0) {
		if (bps == 8) {
			/* Convert from 8 bit to 16 bit */
			int8_t* sb8 = (int8_t*)sbuf;
			for (i = got - 1; i >= 0; --i) {
				sbuf[i] = sb8[i] << 8;
			}
		}
		for (i = 0; i < (int)got; i += channels) {
			printf("%8.8f ", (double)t / (double)srate);
			for (c = 0; c < channels; c++) {
				printf("% .4f ", 1.0 * sbuf[c + i] / 32768.0);
			}
			printf("\n");
			fflush(stdout);
			t++;
		}
	}
}

int main(int argc, char * const * argv)
{
	int channels, srate, ch, bits;

	channels = 1;
	bits	 = 16;
	while ((ch = getopt(argc, argv, "b:c:")) != -1) {
                switch (ch) {
		case 'b': 
			bits = atoi(optarg);
			break;
                case 'c':
                        channels = atoi(optarg);
                        break;
                case '?':
                default:
                        usage();
                }
	}

	argc -= optind;
	argv += optind;

	if (argc != 1)
                usage();

	if (bits != 8 && bits != 16) {
		fprintf(stderr, "bits must be 8 or 16.");
		usage();
	}

        srate = atoi(argv[0]);
	if (srate == 0 || srate > 48000) {
                fprintf(stderr, "smpl_rate must be in range 1--48000");
                usage();
        }

	timeline(srate, channels, bits);

	return 0;
}
