#!/bin/sh

upsample1() {

    exec 2>/dev/null

    print "Generating data..."

    ./tonegen -c 2 212 44100 | dd count=8 | ./timeline -c 2 44100 > ref.dat

    ./tonegen -c 2 212 8000 | dd count=8 | 				      \
	./feeder_rate_fbsd -c 2 8000 44100 | ./timeline -c 2 44100 > cur.dat

    ./tonegen -c 2 212 8000 | dd count=8 | 				      \
	./feeder_rate_new -c 2 8000 44100 | ./timeline -c 2 44100 > new.dat

    print "done\n"

    exec 2>&1

    cat > plot.dat <<endplot
	set key below
	set xrange [0:0.01]
	set title "Upsampling 212Hz tone at 8kHz to 44.1kHz"
	set data style l
	plot "ref.dat" t "Reference (@44.1kHz)", 			      \
	     "cur.dat" t "Current",					      \
	     "new.dat" t "Proposed"

	print "Upsampling 212Hz tone at 8kHz to 44.1kHz"
	pause -1 "Press any key to continue"
	set data style lp
	set xrange [0:0.0014]
	repl

	print "Notice the existing code proceeds in steps, typically 5 or 6 samples long, "
	print "whereas the proposed code interpolates smoothly."
	print ""

	pause -1 "Press any key to continue"
endplot
    gnuplot "plot.dat"
    rm -f ref.dat cur.dat new.dat plot.dat
}

downsample1() {

    exec 2>/dev/null

    print "Generating data..."

    ./tonegen -c 2 212 16000 | dd count=120 | ./timeline -c 2 16000 > ref.dat

    ./tonegen -c 2 212 16000 | dd count=120 | 				      \
	./feeder_rate_fbsd -c 2 16000 11025 | ./timeline -c 2 11025 > cur.dat

    ./tonegen -c 2 212 16000 | dd count=120 | 				      \
	./feeder_rate_new -c 2 16000 11025 | ./timeline -c 2 11025 > new.dat

    print "done\n"

    exec 2>&1

    cat > plot.dat <<-endplot
	set key below
	set xrange [0:0.01]
	set title "Downsampling 212Hz tone from 16kHz to 8kHz"
	set data style lp
	plot "ref.dat" t "Reference (@16kHz)", 				      \
	     "cur.dat" t "Current",					      \
	     "new.dat" t "New"

	print "Downsampling 212Hz tone from 16kHz to 8kHz."
	print ""
	print "Notice current code tracks new code until 0.06 seconds, when"
	print "a sample has been lost"
	print ""
	pause -1 "Press any key to continue"
	set xrange [0.02:0.03]
	repl
	print ""
	print "As we progress forwards current code slips further back.  It"
	print "is not generating enough samples."
	print ""
	print "Also notice kink shortly after 0.028 seconds."
	print ""
	pause -1 "Press any key to continue"
	set xrange [0.0503:0.0608]
	repl
	print ""
	print "And so on. Notice current interpolator does not follow path"
	print "of reference signal very well, region between 0.050s and 0.052s"
	print "looks decidely out and there is another kink later around"
	print "0.057s."
	print ""
	pause -1 "Press any key to continue"
endplot
    gnuplot "plot.dat"
    rm -f ref.dat cur.dat new.dat plot.dat
}

upsample1
downsample1

